package com.sxc.etaxincome.common.util;

import com.github.rholder.retry.*;
import com.google.common.base.Predicates;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.extern.slf4j.Slf4j;

import java.util.concurrent.Callable;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.TimeUnit;

/**
 * @author caosongqing
 * @date 2023/1/16
 * @description retry util.
 */
@Slf4j
@Data
@NoArgsConstructor
@AllArgsConstructor
@Builder
public class RetryUtil<T> {

    private int times = 2;

    private int waitSeconds = 10;

    public T retry(Callable<T> callable) throws ExecutionException, RetryException {
        //定义重试机制
        Retryer<T> retryer = RetryerBuilder.<T>newBuilder()
                //retryIf 重试条件
                .retryIfException()
                .retryIfRuntimeException()
                .retryIfExceptionOfType(Exception.class)
                .retryIfException(Predicates.equalTo(new Exception()))

                //等待策略：每次请求间隔1s
                //fixedWait 固定X秒后重试
                //noWait不等时间直接重试
                //incrementingWait 第一个参数为第一次重试时间，后面会通过设置间隔递增秒数重试
                //randomWait 随机等待设置范围内的时间重试
                .withWaitStrategy(WaitStrategies.fixedWait(waitSeconds, TimeUnit.SECONDS))

                //停止策略 : 尝试请求n次
                .withStopStrategy(StopStrategies.stopAfterAttempt(times))
                .withRetryListener(new TimesRetryListener())
                .withRetryListener(new ExceptionRetryListener())
                .build();
        //利用重试器调用请求
        return retryer.call(callable);
    }

    private class TimesRetryListener implements RetryListener {
        @Override
        public <V> void onRetry(Attempt<V> attempt) {
            log.info("try {} times", attempt.getAttemptNumber());
        }
    }

    private class ExceptionRetryListener implements RetryListener {
        @Override
        public <V> void onRetry(Attempt<V> attempt) {
            if (attempt.hasException()) {
                log.error("error:", attempt.getExceptionCause());
            }
        }
    }
}
